# -*- coding: utf-8 -*-
import sys
if sys.version_info < (3, 0):
    import os
    import pep8
    CURRENT_DIR = os.path.dirname(os.path.abspath(__file__))
    BASE_DIR = os.path.dirname(CURRENT_DIR)

    def test_pep8():
        if pep8.__version__ >= '1.3':
            arglist = [['statistics', True],
                       ['show-source', True],
                       ['repeat', True],
                       ['exclude', []],
                       ['paths', [BASE_DIR]]]

            pep8style = pep8.StyleGuide(arglist,
                                        parse_argv=False,
                                        config_file=True)
            options = pep8style.options
            if options.doctest:
                import doctest
                fail_d, done_d = doctest.testmod(report=False,
                                                 verbose=options.verbose)
                fail_s, done_s = selftest(options)
                count_failed = fail_s + fail_d
                if not options.quiet:
                    count_passed = done_d + done_s - count_failed
                    print("%d passed and %d failed." % (count_passed,
                                                        count_failed))
                    print("Test failed." if count_failed else "Test passed.")
                if count_failed:
                    sys.exit(1)
            if options.testsuite:
                init_tests(pep8style)
            report = pep8style.check_files()
            if options.statistics:
                report.print_statistics()
            if options.benchmark:
                report.print_benchmark()
            if options.testsuite and not options.quiet:
                report.print_results()
            if report.total_errors:
                if options.count:
                    sys.stderr.write(str(report.total_errors) + '\n')
                sys.exit(1)
            # reporting errors (additional summary)
            errors = report.get_count('E')
            warnings = report.get_count('W')
            message = 'pep8: %d errors / %d warnings' % (errors, warnings)
            print(message)
            assert report.total_errors == 0, message
        else:
            # under pep8 1.3
            arglist = [
                '--statistics',
                '--filename=*.py',
                '--show-source',
                '--benchmark',
                '--repeat',
                '--show-pep8',
                #'--qq',
                #'-v',
                BASE_DIR, ]

            options, args = pep8.process_options(arglist)
            runner = pep8.input_file

            for path in args:
                if os.path.isdir(path):
                    pep8.input_dir(path, runner=runner)
                elif not pep8.excluded(path):
                    options.counters['files'] += 1
                    runner(path)

            pep8.print_statistics()
            errors = pep8.get_count('E')
            warnings = pep8.get_count('W')
            message = 'pep8: %d errors / %d warnings' % (errors, warnings)
            print(message)
            assert errors + warnings == 0, message



# -*- coding: utf-8 -*-
"""
    Copyright (C) 2013, 2014 Kouhei Maeda <mkouhei@palmtb.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import os.path
import glob
import sys
from datetime import datetime
from swiftsc import client
from backup2swift import utils

ROTATE_LIMIT = 10


class Backup(object):
    def __init__(self, auth_url, username, password, rotate_limit=ROTATE_LIMIT,
                 verify=True, tenant_id=None, container_name=utils.FQDN):
        self.verify = verify
        (self.token,
         self.storage_url) = client.retrieve_token(auth_url,
                                                   username,
                                                   password,
                                                   tenant_id,
                                                   verify=self.verify)
        if isinstance(rotate_limit, str):
            self.rotate_limit = int(rotate_limit)
        else:
            self.rotate_limit = rotate_limit
        self.container_name = container_name

    def backup(self, target_path):
        """

        Argument:
            target_path: path of backup target file or directory
        """
        if isinstance(target_path, list):
            # for multiple arguments
            [utils.multiprocess(self.backup, path) for path in target_path]
        elif os.path.isdir(target_path):
            [utils.multiprocess(self.backup_file, f)
             for f in glob.glob(os.path.join(target_path, '*'))]
        elif os.path.isfile(target_path):
            self.backup_file(target_path)
        return True

    def backup_file(self, filename, data=None):
        """

        Argument:
            filename: path of backup target file
            data:     backup target file content from stdin pipe
        """
        object_name = os.path.basename(filename)
        if not client.is_container(self.token, self.storage_url,
                                   self.container_name, verify=self.verify):
            # False is no container
            rc = client.create_container(self.token,
                                         self.storage_url,
                                         self.container_name,
                                         verify=self.verify)
            if not (rc == 201 or rc == 202):
                # 201; Created, 202; Accepted
                raise RuntimeError('Failed to create the container "%s"'
                                   % self.container_name, verify=self.verify)

        objects_list = [object.get('name') for object in
                        client.list_objects(self.token,
                                            self.storage_url,
                                            self.container_name,
                                            verify=self.verify)]

        if filename and data:
            # from stdin pipe
            object_name = filename
            filename = data

        if object_name in objects_list:
            self.rotate(filename, object_name, objects_list)
        else:
            rc = client.create_object(self.token,
                                      self.storage_url,
                                      self.container_name,
                                      filename,
                                      object_name=object_name,
                                      verify=self.verify)
            if not (rc == 201 or rc == 202):
                raise RuntimeError('Failed to create the object "%s"'
                                   % object_name)
        return True

    def rotate(self, filename, object_name, objects_list):
        """

        Arguments:
            filename:     filename of backup target
            object_name:  name of object on Swift
            objects_list: list of objects on Swift
        """
        # copy current object to new object
        timestamp = datetime.now().strftime("%Y%m%d-%H%M%S")
        new_object_name = object_name + '_' + timestamp
        rc = client.copy_object(self.token, self.storage_url,
                                self.container_name, object_name,
                                new_object_name, verify=self.verify)
        if rc != 201:
            raise RuntimeError('Failed to copy object "%s"' % new_object_name)

        # create new object
        rc = client.create_object(self.token, self.storage_url,
                                  self.container_name, filename,
                                  object_name=object_name,
                                  verify=self.verify)
        if rc != 201:
            raise RuntimeError('Failed to create the object "%s"'
                               % object_name)

        # delete old objects
        archive_list = [obj for obj in objects_list
                        if obj.startswith(object_name + '_')]
        archive_list.reverse()
        [utils.multiprocess(client.delete_object, self.token,
                            self.storage_url, self.container_name,
                            obj, verify=self.verify)
         for i, obj in enumerate(archive_list)
         if i + 1 > self.rotate_limit - 1]
        return True

    def retrieve_backup_data_list(self, verbose=False):
        """

        Argument:
            verbose: boolean flag of listing objects
        """
        if not client.is_container(self.token, self.storage_url,
                                   self.container_name, verify=self.verify):
            return []

        if verbose:
            backup_l = [i for i in client.list_objects(self.token,
                                                       self.storage_url,
                                                       self.container_name,
                                                       verify=self.verify)]
        else:
            backup_l = [i.get('name') for i
                        in client.list_objects(self.token,
                                               self.storage_url,
                                               self.container_name,
                                               verify=self.verify)]
        return backup_l

    def retrieve_backup_data(self, object_name, output_filepath=None):
        """

        Argument:
            object_name: delete target object name
        """
        if isinstance(object_name, list):
            # for retrieve multiple objects
            output_filepath = None
            [utils.multiprocess(self.retrieve_backup_data, obj)
             for obj in object_name]
        elif (client.is_container(self.token, self.storage_url,
                                  self.container_name, verify=self.verify) and
              client.is_object(self.token, self.storage_url,
                               self.container_name, object_name,
                               verify=self.verify)):
            rc, content = client.retrieve_object(self.token,
                                                 self.storage_url,
                                                 self.container_name,
                                                 object_name,
                                                 verify=self.verify)
            if not rc:
                raise RuntimeError('Failed to retrieve the object "%s"'
                                   % object_name)
            if output_filepath:
                fpath = os.path.abspath(output_filepath)
                dpath = os.path.dirname(fpath)
                if not os.path.isdir(dpath):
                    raise IOError('No such directory "%s"' % dpath)
            else:
                dpath = os.path.abspath(os.curdir)
                fpath = os.path.join(dpath, object_name)
            if sys.version_info > (3, 0) and isinstance(content, bytes):
                mode = 'bw'
            else:
                mode = 'w'
            with open(fpath, mode) as f:
                f.write(content)
        else:
            raise RuntimeError('No such object "%s"' % object_name)

    def delete_backup_data(self, object_name):
        """

        Argument:
            object_name: delete target object name
        """
        if isinstance(object_name, list):
            # for multiple arguments
            [self.delete_backup_data(obj) for obj in object_name]

        elif (client.is_container(self.token, self.storage_url,
                                  self.container_name, verify=self.verify) and
              client.is_object(self.token, self.storage_url,
                               self.container_name, object_name,
                               verify=self.verify)):
            rc = client.delete_object(self.token,
                                      self.storage_url,
                                      self.container_name,
                                      object_name,
                                      verify=self.verify)
            if not rc == 204:
                raise RuntimeError('Failed to delete the object "%s"'
                                   % object_name)
            return True
        else:
            raise RuntimeError('No such object "%s"' % object_name)



# -*- coding: utf-8 -*-
"""
    Copyright (C) 2013, 2014 Kouhei Maeda <mkouhei@palmtb.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import argparse
import os
import os.path
import sys
from backup2swift import __version__, backup, utils, config

DEFAULT_CONF = '.bu2sw.conf'


def parse_options():
    parser = argparse.ArgumentParser(description='usage')
    setoption(parser, 'version')
    setoption(parser, 'config')
    setoption(parser, 'command')
    setoption(parser, 'verbose')
    args = parser.parse_args()
    return args


def setoption(parser, keyword):
    """

    Arguments:
        parser: object of argparse
        keyword: switching keyword
    """
    if keyword == 'version':
        parser.add_argument('-V', '--version', action='version',
                            version=__version__)
    elif keyword == 'config':
        parser.add_argument('-c', '--config', action='store',
                            help='configuraton file of backup2swift')
    elif keyword == 'verbose':
        parser.add_argument('-v', '--verbose', action='store_true',
                            help='list verbose')
        parser.add_argument('-o', '--output', action='store',
                            help=('specify filename of retrieved data'
                                  ' (only retrieving simple object)'))
    elif keyword == 'command':
        parser.add_argument('-C', '--container', action='store',
                            help=('specify container name (default: '
                                  'FQDN of host when executes this command)'))
        group = parser.add_mutually_exclusive_group(required=True)
        group.add_argument('-l', '--list', action='store_true',
                           help='listing object data')
        group.add_argument('-p', '--path', action='store', nargs='+',
                           help='target files/dir path of backup')
        group.add_argument('-s', '--stdin', action='store',
                           help='backup fromstdin pipe & specify object name')
        group.add_argument('-d', '--delete', action='store', nargs='+',
                           help='delete backup data')
        group.add_argument('-r', '--retrieve', action='store', nargs='+',
                           help='retrieve backup data')
        parser.set_defaults(func=execute_swift_client)


def check_config_file(args_config):
    """

    Argument:
        args_config: args.config
    """
    if args_config:
        # override configuration file path
        config_file = args_config
    elif os.path.isfile(os.path.join(os.environ['HOME'], DEFAULT_CONF)):
        # use default configuration file
        config_file = os.path.join(os.environ['HOME'], DEFAULT_CONF)
    else:
        raise IOError(('Setup "~/.bu2sw.conf" or '
                       'specify configuration file with "-c" option'))
    return config_file


def execute_swift_client(args):
    """

    Argument:
        args: argument object
    """
    config_file = check_config_file(args.config)
    (auth_url, username, password, rotate_limit,
     verify, tenant_id) = config.check_config(config_file)
    if args.container:
        container_name = args.container
    else:
        container_name = utils.FQDN
    b = backup.Backup(auth_url, username, password, rotate_limit,
                      verify=verify, tenant_id=tenant_id,
                      container_name=container_name)
    if args.list:
        # listing backup data
        backup_l = b.retrieve_backup_data_list(args.verbose)
        utils.list_data(backup_l)
    elif args.path:
        # backup data to swift
        b.backup(args.path)
    elif args.stdin:
        # backup via stdin pipe
        if sys.version_info > (3, 0):
            # for python3
            b.backup_file(args.stdin, data=sys.stdin.buffer.raw)
        else:
            # for python2
            b.backup_file(args.stdin, data=sys.stdin)
    elif args.retrieve:
        # retrive backup data
        b.retrieve_backup_data(args.retrieve, args.output)
    elif args.delete:
        # delete backup data
        b.delete_backup_data(args.delete)


def main():
    try:
        args = parse_options()
        args.func(args)
    except (RuntimeError, IOError) as error:
        # syslog.ERR is 3
        utils.logging(3, error)

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""
    Copyright (C) 2013, 2014 Kouhei Maeda <mkouhei@palmtb.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import unittest
import sys
import os.path
sys.path.append(os.path.abspath('src'))
import backup2swift.config as c


class ConfigTests(unittest.TestCase):

    def test_check_config(self):
        self.assertEqual(('https://example.org/auth/v1.0',
                         'username',
                         'password',
                         '10',
                          True,
                          None),
                         c.check_config('examples/bu2sw.conf'))

    def test_check_config_ignore(self):
        self.assertEqual(('https://example.org/auth/v1.0',
                         'username',
                         'password',
                         '10',
                          False,
                          None),
                         c.check_config('examples/bu2sw_ignore_verify.conf'))



# -*- coding: utf-8 -*-
import os.path

auth_url = 'https://example.org/auth/v1.0'
username = 'username'
password = 'password'
token = 'hoge'
s_url = 'fuga'
container_name = 'moge'
objects = [{'bytes': 5246,
            'content_type': 'text/plain',
            'hash': 'd226a3f396cbe3c187f2b7b78030eebb',
            'last_modified': '2013-05-01T08:40:00.459930',
            'name': 'sample.txt'},
           {'bytes': 5246,
            'content_type': 'text/plain',
            'hash': 'd226a3f396cbe3c187f2b7b78030eebb',
            'last_modified': '2013-05-01T09:23:40.060560',
            'name': 'sample_2.txt'},
           {'bytes': 5246,
            'content_type': 'text/plain',
            'hash': 'd226a3f396cbe3c187f2b7b78030eebb',
            'last_modified': '2013-05-01T09:23:26.092580',
            'name': 'sample_3.txt'}]
objects_name_l = [i.get('name') for i in objects]
test_file = os.path.abspath('src/backup2swift_tests/sample.txt')
test_files = [os.path.abspath('src/backup2swift_tests/sample.txt'),
              os.path.abspath('src/backup2swift_tests/sample2.txt')]
object_name = 'sample.txt'
objects_name = ['sample.txt', 'sample2.txt']
objects_header = ['bytes', 'content_type', 'hash', 'last_modified', 'name']
header_width = [5, 12, 4, 13, 4]
objects_row_width = [5, 12, 32, 26, 12]
dummy_row_width = [5, 12, 32, 26, 11]
config_file = 'examples/bu2sw.conf'



# -*- coding: utf-8 -*-
"""
    Copyright (C) 2013, 2014 Kouhei Maeda <mkouhei@palmtb.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import unittest
import sys
import os
sys.path.append(os.path.abspath('src'))
import backup2swift.command as c
import backup2swift_tests.test_vars as v


class CommandTests(unittest.TestCase):

    def test_check_config_file(self):
        self.assertEqual(v.config_file,
                         c.check_config_file(v.config_file))

    def test_check_config_file_fail(self):
        if not os.path.isfile(os.path.join(os.environ['HOME'], '.bu2sw.conf')):
            self.assertRaises(IOError, c.check_config_file, None)
